@@ -5,10 +5,10 @@ import qualified Tests.Vector.UnitTests
 import qualified Tests.Bundle
 import qualified Tests.Move
 
-import Test.Framework (defaultMain)
+import Test.Tasty (defaultMain,testGroup)
 
 main :: IO ()
-main = defaultMain $ Tests.Bundle.tests
+main = defaultMain $ testGroup "toplevel" $ Tests.Bundle.tests
                   ++ Tests.Vector.tests
                   ++ Tests.Vector.UnitTests.tests
                   ++ Tests.Move.tests
